<?php

namespace Drupal\social_auth_yandex\Service;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Url;
use Drupal\social_auth\AuthManager\OAuth2Manager;
use Symfony\Component\HttpFoundation\RequestStack;
use Yandex\OAuth\OAuthClient;
use GuzzleHttp\Client;
use Drupal\Component\Serialization\Json;

/**
 * Contains all the logic for Yandex OAuth2 authentication.
 */
class YandexAuthManager extends OAuth2Manager {

  /**
   * Authenticated user email.
   *
   * @var string
   */
  protected $email;

  /**
   * Authenticated user id.
   *
   * @var string
   */
  protected $userId;

  /**
   * The OAuthClient client.
   *
   * @var Yandex\OAuth\OAuthClient
   */
  protected $client;

  /**
   * The currently active request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The secret state for the authentication.
   *
   * @var string
   */
  protected $state;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   Used for accessing configuration object factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Used to get current request information.
   */
  public function __construct(ConfigFactory $configFactory,
                              LoggerChannelFactoryInterface $logger_factory,
                              RequestStack $request_stack) {
    parent::__construct($configFactory->get('social_auth_yandex.settings'), $logger_factory);
    $this->request = $request_stack->getCurrentRequest();
  }

  /**
   * {@inheritdoc}
   */
  public function authenticate() {
    $client_id = $this->settings->get('client_id');
    $client_secret = $this->settings->get('client_secret');
    $client = new OAuthClient($client_id, $client_secret);
    try {
      $code = $this->request->query->get('code');
      $token = $client->requestAccessToken($code)->getAccessToken();
      $this->setAccessToken($token);
    }
    catch (\Exception $e) {
      $msg = $e->getMessage();
      $this->loggerFactory->get('social_auth_yandex')
        ->error("There was an error during authentication. Exception: $msg");
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getUserInfo() {

    if (!$this->user) {
      $token = $this->getAccessToken();
      $client = new Client();
      $res = $client->request('GET', 'https://login.yandex.ru/info', [
        'headers' => [
          'Authorization' => "OAuth $token",
          'Accept'     => 'application/json',
        ],
        'query' => [
          'format' => 'json',
        ],
      ]);
      $code = $res->getStatusCode();
      if ($res->getStatusCode() == 200) {
        $json = $res->getBody();
        $data = Json::decode($json);
        $this->user = [
          'id' => $data['id'],
          'login' => $data['login'],
          'client_id' => $data['client_id'],
          'email' => $data['default_email'],
          'real_name' => $data['real_name'],
          'first_name' => $data['first_name'],
          'last_name' => $data['last_name'],
        ];
        if (!empty($data['default_avatar_id'])) {
          $this->user['picture_url'] = "https://avatars.yandex.net/get-yapic/{$data['default_avatar_id']}/islands-200";
        }
      }

      return $this->user;
    }

    return $this->user;
  }

  /**
   * {@inheritdoc}
   */
  public function getAuthorizationUrl() {
    $id = $this->getClientId();
    $state = $this->getState();
    $client = new OAuthClient($id);
    $scopes = [
      'login:email',
      'login:info',
    ];
    return $client->getAuthUrl(OAuthClient::CODE_AUTH_TYPE, $state);
  }

  /**
   * {@inheritdoc}
   */
  public function requestEndPoint($method, $path, $domain = NULL, array $options = []) {
    // TODO: Implement this method.
  }

  /**
   * {@inheritdoc}
   */
  public function getState() {
    if (!$this->state) {
      $chars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
      $length = 32;

      // Generates a random string of 32 characters.
      $this->state = substr(str_shuffle(str_repeat($chars, ceil($length / strlen($chars)))), 1, $length);
    }

    return $this->state;
  }

  /**
   * Get client id.
   *
   * @return string
   *   Client client id.
   */
  public function getClientId() {
    return $this->settings->get('client_id');
  }

  /**
   * Get client secret key.
   *
   * @return string
   *   Client secret key.
   */
  public function getClientSecret() {
    return $this->settings->get('client_secret');
  }

  /**
   * Get redirect url.
   *
   * @return string
   *   The redirect url.
   */
  public function getRedirectUrl() {
    return Url::fromRoute('social_auth_yandex.callback')->setAbsolute()->toString();
  }

}
